package hr.fer.zemris.diplomskiseminar.mmo.algorithms;

import hr.fer.zemris.diplomskiseminar.mmo.datamodel.DecimalArraySolution;
import hr.fer.zemris.diplomskiseminar.mmo.datamodel.Front;

import java.util.List;
import java.util.Random;

public abstract class GenericAlgorithm {

	public final double lamda = 0.2;

	protected void crossover(DecimalArraySolution solution1,
			DecimalArraySolution solution2, DecimalArraySolution child1,
			DecimalArraySolution child2, Random rand, double mins, double maxs, int n) {
		
		double[] H1 = child1.getVector();
		double[] H2 = child2.getVector();
		double[] P1 = solution1.getVector();
		double[] P2 = solution2.getVector();
		
		for(int i=0; i<n; i++){
			H1[i] = lamda*P1[i] + (1-lamda)*P2[i];
			if(H1[i] < mins){
				H1[i] = mins;
			}
			if(H1[i] > maxs){
				H1[i] = maxs;
			}
			H2[i] = lamda*P2[i] + (1-lamda)*P1[i];
			if(H2[i] < mins){
				H2[i] = mins;
			}
			if(H2[i] > maxs){
				H2[i] = maxs;
			}
		}
	}
	
	protected void mutate(DecimalArraySolution child1, Random rand,
			double mutProb, double mins, double maxs, int n) {
		
		if(rand.nextDouble()>mutProb){
			return;
		}
		double[] v = child1.getVector();
		for(int i=0; i<n; i++){
			while(true){
				double val = rand.nextGaussian()*(maxs - mins)/2.0;
				if(val + v[i]<maxs && val + v[i]>mins){
					v[i] += val;
					break;
				}
			}
			 
		}
	}
	
	public abstract List<Front> start();
}
