package hr.fer.zemris.diplomskiseminar.mmo.algorithms;

import hr.fer.zemris.diplomskiseminar.mmo.datamodel.DecimalArraySolution;
import hr.fer.zemris.diplomskiseminar.mmo.datamodel.Front;
import hr.fer.zemris.diplomskiseminar.mmo.datamodel.Parameters;
import hr.fer.zemris.diplomskiseminar.mmo.problems.IMMOProblem;

import java.util.ArrayList;
import java.util.List;

public class PredatorPrey extends GenericAlgorithm{
	private int POP_SIZE;
	private int matrixDimension;
	private IMMOProblem problem;
	private Predator[] predators;
	private int ITER_NUM;
	private double MUT_PROB;
	
	public PredatorPrey(int POP_SIZE, IMMOProblem problem, int ITER_NUM, double mutProb) {
		this.POP_SIZE = POP_SIZE;
		this.problem = problem;
		this.matrixDimension = (int) Math.floor(Math.sqrt(this.POP_SIZE));
		this.ITER_NUM = ITER_NUM;
		this.MUT_PROB = mutProb;
		predators = new Predator[problem.objectivesNumber()];
	}
	
	public List<Front> start(){
		
		for(int i=0; i<predators.length; i++){
			predators[i] = new Predator(i);
		}
		
		Graph graph = new Graph();
		graph.initialize();
		int iter = 0;
		while(iter < ITER_NUM){
			System.out.println("Iteracija "+iter);
			for(Predator predator : predators){
				
				int x = predator.getXLocation();
				int y = predator.getYLocation();
				Prey prey = graph.getPrey(x, y);
				List<Prey> neighbours = graph.getNeighbours(x, y);
				neighbours.add(prey);
				int eaten = predator.choosePrey(neighbours);
				Prey victim = neighbours.get(eaten);
				Prey[] parents = predator.chooseBest(neighbours);
				Prey child1 = new Prey(problem.objectivesNumber(), problem.getMinBound(), problem.getMaxBound(), problem.getDimensionNumber(), x, y);
				Prey child2 = new Prey(problem.objectivesNumber(), problem.getMinBound(), problem.getMaxBound(), problem.getDimensionNumber(), x, y);
				crossover(parents[0], parents[1], child1, child2, Parameters.rand, problem.getMinBound(), problem.getMaxBound(), problem.getDimensionNumber());
				Prey newborn;
				if(Parameters.rand.nextBoolean()){
					newborn = child1;
				}
				else{
					newborn = child2;
				}
				newborn.rank=0;
				newborn.setValue(problem);
				mutate(newborn, Parameters.rand, MUT_PROB, problem.getMinBound(), problem.getMaxBound(), problem.getDimensionNumber());
				graph.setPrey(newborn, victim.getXLoc(), victim.getYLoc());
				predator.move();
			}
			iter++;
		}
		List<DecimalArraySolution> finalSolutions = graph.getAllPreys();
		Front front = new Front();
		front.solutions = finalSolutions;
		List<Front> list = new ArrayList<>();
		list.add(front);
		return list;
	}
	
	public Prey[] getParents(){
		return null;
	}

	private class Graph{
		
		private Prey[][] graphPrey = new Prey[matrixDimension][matrixDimension];
		
		public Graph() {
			
		}
		
		public void initialize(){
			
			for(int x=0; x<matrixDimension; x++){
				for(int y=0; y<matrixDimension; y++){
					graphPrey[x][y] = new Prey(problem.objectivesNumber(), problem.getMinBound(), problem.getMaxBound(), problem.getDimensionNumber(), x, y);
					graphPrey[x][y].randomize(Parameters.rand);
					graphPrey[x][y].setValue(problem);
					graphPrey[x][y].rank = 0;
				}
			}
		}
		
		public List<DecimalArraySolution> getAllPreys(){
			List<DecimalArraySolution> all = new ArrayList<>();
			for(int i=0; i<matrixDimension; i++){
				for(int j=0; j<matrixDimension; j++){
					all.add(graphPrey[i][j]);
				}
			}
			return all;
		}
		
		public Prey getPrey(int x, int y){
			return graphPrey[x][y];
		}
		
		
		public void setPrey(Prey prey, int x, int y){
			prey.xLoc = x;
			prey.yLoc = y;
			graphPrey[x][y] = prey;
		}
		
		public List<Prey> getNeighbours(int xPos, int yPos){
			List<Prey> preys = new ArrayList<>();
			int x = 0;
			int y = 0;
			for(int i=0; i<8; i++){
				switch(i){
					case 0: x = xPos-1;
							y = yPos-1;
							break;
					case 1: y = yPos-1;
							break;
					case 2: x = xPos+1;
							y = yPos-1;
							break;
					case 3: x = xPos+1;
							break;
					case 4: x = xPos+1;
							y = yPos+1;
							break;
					case 5: y = yPos+1;
							break;
					case 6: x = xPos-1;
							y = yPos+1;
							break;
					case 7: x = xPos-1;
							break;
					default: break;
				}
				if(x < 0){
					x = matrixDimension-1;
				}
				if(x > matrixDimension-1){
					x = 0;
				}
				if(y < 0){
					y = matrixDimension-1;
				}
				if(y > matrixDimension-1){
					y = 0;
				}
				preys.add(graphPrey[x][y]);
			}
			return preys;
		}
	}
	
	private class Predator{
		
		private int xLocation;
		private int yLocation;
		private int number;
		
		public Predator(int number) {
			xLocation = Parameters.rand.nextInt(matrixDimension);
			yLocation = Parameters.rand.nextInt(matrixDimension);
			this.number = number;
		}
		
		public int getXLocation(){
			return xLocation;
		}
		
		public int getYLocation(){
			return yLocation;
		}
		
		public int choosePrey(List<Prey> preys){
			int index = 0;
			double maxValue = preys.get(index).getValue(number);
			for(int i=1; i<preys.size(); i++){
				Prey prey = preys.get(i);
				double valP = prey.getValue(number);
				if(valP > maxValue){
					maxValue = valP;
					index = i;
				}
				
			}
			return index;
		}
		
		public Prey[] chooseBest(List<Prey> preys){
			
			int index1 = 0;
			double minValue = preys.get(index1).getValue(number);
			for(int i=1; i<preys.size(); i++){
				Prey prey = preys.get(i);
				double valP = prey.getValue(number);
				if(valP < minValue){
					minValue = valP;
					index1 = i;
				}				
			}
			
			int index2 = 0;
			minValue = preys.get(index2).getValue(number);
			for(int i=1; i<preys.size(); i++){
				if(i!=index1){
					Prey prey = preys.get(i);
					double valP = prey.getValue(number);
					if(valP < minValue){
						minValue = valP;
						index2 = i;
					}
				}								
			}
			Prey[] res = new Prey[2];
			res[0] = preys.get(0);
			res[1] = preys.get(1);
			return res;
		}
		
		public void move(){
			int x = xLocation;
			int y = yLocation;
			int maxX = matrixDimension-1;
			int maxY = matrixDimension-1;
			int random = Parameters.rand.nextInt(8);
			switch(random){
				case 0: x = x-1;
						y = y-1;
						break;
				case 1: y = y-1;
						break;
				case 2: x = x+1;
						y = y-1;
						break;
				case 3: x = x+1;
						break;
				case 4: x = x+1;
						y = y+1;
						break;
				case 5: y = y+1;
						break;
				case 6: x = x-1;
						y = y+1;
						break;
				case 7: x = x-1;
						break;
				default: break;
		
			}
			if(x < 0){
				x = maxX;
			}
			if(x > maxX){
				x = 0;
			}
			if(y < 0){
				y = maxY;
			}
			if(y > maxY){
				y = 0;
			}
			this.xLocation = x;
			this.yLocation = y;
		}
	}
	
	private class Prey extends DecimalArraySolution{
		
		private int xLoc;
		private int yLoc;

		public Prey(int M, double mins, double maxs, int n, int x, int y) {
			super(M, mins, maxs, n);
			this.xLoc = x;
			this.yLoc = y;
		}
		
		public Prey(int M, double[] mins, double[] maxs, double[] vector, int n){
			super(M, mins, maxs, vector, n);
		}	
		
		public int getXLoc(){
			return xLoc;
		}
		
		public int getYLoc(){
			return yLoc;
		}
	}
}
