package hr.fer.zemris.diplomskiseminar.mmo.datamodel;

import hr.fer.zemris.diplomskiseminar.mmo.problems.IMMOProblem;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class DecimalArraySolution {

	private double[] decimalVector;
	private double[] values;
	public double TOTAL_FITNESS;
	public double lowProbability;
	public double highProbability;
	private int M;
	private final double trashhold = 1E-7;
	public int rank;
	public int ni;
	private List<DecimalArraySolution> Si;

	private double[] mins;
	private double[] maxs;
	private int n;
	
	public DecimalArraySolution(int M, double mins, double maxs, int n) {
		
		this.M = M;
		this.mins = new double[n];
		this.maxs = new double[n];
		for(int i=0; i<n; i++){
			this.mins[i] = mins;
			this.maxs[i] = maxs;
		}
		values = new double[M];
		decimalVector = new double[n];
		rank = -1;
		this.n = n;
	}
	
	public DecimalArraySolution(int M, double[] mins, double[] maxs, double[] vector, int n){
		
		this.n = n;
		this.M = M;
		this.mins = mins;
		this.maxs = maxs;
		values = new double[M];
		rank = -1;
		TOTAL_FITNESS = 0.0;
		ni = 0;
		this.decimalVector = vector;
	}
	
	public DecimalArraySolution duplicate(){
		double[] doubleData = new double[n];
		System.arraycopy(decimalVector, 0, doubleData, 0, decimalVector.length);
		DecimalArraySolution duplicate = new DecimalArraySolution(M, mins, maxs, doubleData, n);
		duplicate.rank = -1;
		duplicate.ni = 0;
		duplicate.Si = new ArrayList<>();
		duplicate.TOTAL_FITNESS = 0;
		return duplicate;
	}
	
	public void randomize(Random rand){
		
		for(int i=0; i<n; i++){
			decimalVector[i] =  mins[i] + (maxs[i] - mins[i]) * rand.nextDouble();
		}
	}
	
	public int dominates(DecimalArraySolution o){
		
		boolean thisDominates = false;
		boolean oDominates = false;
		boolean same = true;
		
		for(int i=0; i<values.length; i++){
			if(Math.abs(values[i] - o.getValue(i)) > trashhold){
				same = false;
			}
		}
		if(same){
			return 0;
		}
		for(int i=0; i<values.length; i++){
			if(values[i] < o.getValue(i) && Math.abs(values[i] - o.getValue(i)) > trashhold){
				thisDominates = true;
			}
			else if(values[i] > o.getValue(i) && Math.abs(values[i] - o.getValue(i)) > trashhold){
				oDominates = true;
			}
		}
		if(thisDominates && oDominates){
			return 0;
		}
		if(thisDominates){
			return 1;
		}
		
		return -1;
	}
	
	public void setValue(IMMOProblem problem){
		for(int i=0; i<M; i++){
			values[i] = problem.getObjective(i).getFunctionValue(decimalVector);
		}
	}
	
	public double getDistance(DecimalArraySolution o, double[] minValues, double[] maxValues){
		
		double[] vector2 = o.getVector();
		double sum = 0.0;
		for(int k=0; k<decimalVector.length; k++){
			sum += Math.pow(((decimalVector[k] - vector2[k])/(maxValues[k] - minValues[k])), 2.0);
		}
		double distance = Math.sqrt(sum);
		return distance;
	}
	
	/*
	 * public double getDistance(BitvectorSolution o, double[] mins, double[] maxs){
		
		double[] dimensions1 = decoder.decode(this);
		double[] dimensions2 = decoder.decode(o);
		System.out.println("PRVI: "+Arrays.toString(dimensions1));
		System.out.println("PRVI: " + Arrays.toString(bitvector));
		System.out.println("DRUGI:"+Arrays.toString(dimensions1));
		System.out.println("DRUGI:" + Arrays.toString(o.bitvector));
		double sum = 0.0;
		for(int k=0; k<dimensions1.length; k++){
			sum += Math.pow(((dimensions1[k] - dimensions2[k])/(maxs[k] - mins[k])), 2.0);
		}
		System.out.println("SUM " + sum);
		double distance = Math.sqrt(sum);
		System.out.println("DIST " + distance);
		return distance;
	}
	 * */
	
	
	public double getValue(int i){
		return values[i];
	}
	
	public double[] getValues(){
		return values;
	}
	
	public void setValues(double[] values){
		this.values = values;
	}
	
	public double[] getVector(){
		return decimalVector;
	}
	
	public List<DecimalArraySolution> getSi() {
		return Si;
	}

	public void setSi(List<DecimalArraySolution> si) {
		Si = si;
	}
}
