package hr.fer.zemris.diplomskiseminar.mmo.gui;

import hr.fer.zemris.diplomskiseminar.mmo.algorithms.GenericAlgorithm;
import hr.fer.zemris.diplomskiseminar.mmo.algorithms.NSGA;
import hr.fer.zemris.diplomskiseminar.mmo.algorithms.PredatorPrey;
import hr.fer.zemris.diplomskiseminar.mmo.datamodel.DecimalArraySolution;
import hr.fer.zemris.diplomskiseminar.mmo.datamodel.Front;
import hr.fer.zemris.diplomskiseminar.mmo.problems.IMMOProblem;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMOProblemFON;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMOProblemKUR;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMOProblemPOL;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMO_ZDT1;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMO_ZDT2;
import hr.fer.zemris.diplomskiseminar.mmo.problems.MMO_ZDT3;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;
import javax.swing.WindowConstants;

public class MMOGui extends JFrame{

	private static final long serialVersionUID = 1L;
	JRadioButton radioButton_FON;
	JRadioButton radioButton_POL;
	JRadioButton radioButton_KUR;
	JRadioButton radioButton_ZDT1;
	JRadioButton radioButton_ZDT2;
	JRadioButton radioButton_ZDT3;
	
	JRadioButton radioButtonNSGA;
	JRadioButton radioButtonPP;
	
	JTextField textFieldPopSize;
	JTextField textFieldNumIter;
	JTextField textFieldMutProb;
	JTextField textFieldCrossProb;
	JTextField textFieldSigmaShare;
	JTextField textFieldEpsilon;
	JLabel labelCrossProb;
	JLabel labelSS;
	JLabel labelEps;
	
	List<Front> frontsFinalResult = null;
	
	public MMOGui() {
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);		
		setResizable(false);
		setTitle("Diplomski seminar, Mia Primorac");
		initGUI();
		pack();
	}
	
	private void initGUI(){
		this.getContentPane().setLayout(new BorderLayout());
		JPanel panelMain = new JPanel();
		panelMain.setLayout(new BoxLayout(panelMain, BoxLayout.Y_AXIS));
		
		JPanel panelFunkcije = new JPanel();
		panelFunkcije.setLayout(new GridLayout(3, 1, 5, 5));
		panelFunkcije.setBorder(BorderFactory.createTitledBorder("Viekriterijski problemi"));
		ButtonGroup buttonGroupProblems = new ButtonGroup();
		radioButton_FON = new JRadioButton("Fonesca and Fleming's study problem");
		radioButton_POL = new JRadioButton("Poloni's study problem");
		radioButton_KUR = new JRadioButton("Kursawe's study problem");
		radioButton_ZDT1 = new JRadioButton("Zitzler et al. 1");
		radioButton_ZDT2 = new JRadioButton("Zitzler et al. 2");
		radioButton_ZDT3 = new JRadioButton("Zitzler et al. 3");
		radioButton_FON.setSelected(true);
		buttonGroupProblems.add(radioButton_POL);
		buttonGroupProblems.add(radioButton_FON);
		buttonGroupProblems.add(radioButton_KUR);
		buttonGroupProblems.add(radioButton_ZDT1);
		buttonGroupProblems.add(radioButton_ZDT2);
		buttonGroupProblems.add(radioButton_ZDT3);
		panelFunkcije.add(radioButton_FON);
		panelFunkcije.add(radioButton_POL);
		panelFunkcije.add(radioButton_KUR);
		panelFunkcije.add(radioButton_ZDT1);
		panelFunkcije.add(radioButton_ZDT2);
		panelFunkcije.add(radioButton_ZDT3);
		panelMain.add(panelFunkcije);
		
		this.getContentPane().add(panelMain, BorderLayout.CENTER);
		
		JPanel panelAlgoritmi = new JPanel();
		panelAlgoritmi.setLayout(new GridLayout(2, 1, 5, 5));
		panelAlgoritmi.setBorder(BorderFactory.createTitledBorder("Algoritmi"));
		radioButtonNSGA = new JRadioButton(actionNSGASelected);
		radioButtonNSGA.setSelected(true);
		radioButtonPP = new JRadioButton(actionPPSelected);
		panelAlgoritmi.add(radioButtonNSGA);
		panelAlgoritmi.add(radioButtonPP);
		panelMain.add(panelAlgoritmi);
		ButtonGroup buttonGroupAlgorithms = new ButtonGroup();
		buttonGroupAlgorithms.add(radioButtonNSGA);
		buttonGroupAlgorithms.add(radioButtonPP);
		
		JPanel panelParametri = new JPanel();
		panelParametri.setLayout(new GridLayout(6, 2, 5, 5));
		panelParametri.setBorder(BorderFactory.createTitledBorder("Parametri"));
		textFieldCrossProb = new JTextField("0.8");
		textFieldEpsilon = new JTextField("0.5");
		textFieldMutProb = new JTextField("0.05");
		textFieldNumIter = new JTextField("500");
		textFieldPopSize = new JTextField("200");
		textFieldSigmaShare = new JTextField("2");
		panelParametri.add(new JLabel("Veliina populacije:"));
		panelParametri.add(textFieldPopSize);
		panelParametri.add(new JLabel("Broj iteracija:"));
		panelParametri.add(textFieldNumIter);
		panelParametri.add(new JLabel("Vjerojatnost mutacije:"));
		panelParametri.add(textFieldMutProb);
		labelCrossProb = new JLabel("Vjerojatnost krianja:");
		labelEps = new JLabel("Epsilon:");
		labelSS = new JLabel("Sigma-share:");
		panelParametri.add(labelCrossProb);
		panelParametri.add(textFieldCrossProb);
		panelParametri.add(labelSS);
		panelParametri.add(textFieldSigmaShare);
		panelParametri.add(labelEps);
		panelParametri.add(textFieldEpsilon);
		panelMain.add(panelParametri);
		
		JPanel panelControl = new JPanel(new GridLayout(2, 2, 10, 10));
		panelControl.add(new JLabel());
		panelControl.add(new JButton(actionStartAlgorithm));
		panelControl.add(new JLabel());
		panelControl.add(new JButton(actionShowResults));
		panelMain.add(panelControl);
	}
	
	private Action actionNSGASelected = new AbstractAction("NSGA") {
		
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent arg0) {
			textFieldCrossProb.setEnabled(true);
			textFieldEpsilon.setEnabled(true);
			textFieldSigmaShare.setEnabled(true);
			labelCrossProb.setEnabled(true);
			labelEps.setEnabled(true);
			labelSS.setEnabled(true);
		}
	};
	
	private Action actionPPSelected = new AbstractAction("Predator-Prey") {
		
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent arg0) {
			textFieldCrossProb.setEnabled(false);
			textFieldEpsilon.setEnabled(false);
			textFieldSigmaShare.setEnabled(false);
			labelCrossProb.setEnabled(false);
			labelEps.setEnabled(false);
			labelSS.setEnabled(false);
		}
	};
	
	private Action actionStartAlgorithm = new AbstractAction("Start") {
		
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent arg0) {

			IMMOProblem problem = null;
			if(radioButton_KUR.isSelected()){
				problem = new MMOProblemKUR();
			}
			else if(radioButton_POL.isSelected()){
				problem = new MMOProblemPOL();
			}
			else if(radioButton_FON.isSelected()){
				problem = new MMOProblemFON();
			}
			else if(radioButton_ZDT1.isSelected()){
				problem = new MMO_ZDT1();
			}
			else if(radioButton_ZDT2.isSelected()){
				problem = new MMO_ZDT2();
			}
			else if(radioButton_ZDT3.isSelected()){
				problem = new MMO_ZDT3();
			}
			
			GenericAlgorithm algorithm;
			int popSize;
			int numIter;
			double mutProb;
			double crossProb;
			double sigmaShare;
			double epsilon;
			try{
				popSize = Integer.parseInt(textFieldPopSize.getText().trim());
				numIter = Integer.parseInt(textFieldNumIter.getText().trim());
				mutProb = Double.parseDouble(textFieldMutProb.getText().trim());
			}
			catch (NumberFormatException e) {
				JOptionPane.showMessageDialog(MMOGui.this, "Pogreka u ulaznim parametrima!", "Pogreka", JOptionPane.ERROR_MESSAGE);
				return;
			}
			
			if(radioButtonNSGA.isSelected()) {
				try{
					crossProb = Double.parseDouble(textFieldCrossProb.getText().trim());
					sigmaShare = Double.parseDouble(textFieldSigmaShare.getText().trim());
					epsilon = Double.parseDouble(textFieldEpsilon.getText().trim());
				}
				catch (NumberFormatException e) {
					JOptionPane.showMessageDialog(MMOGui.this, "Pogreka u ulaznim parametrima!", "Pogreka", JOptionPane.ERROR_MESSAGE);
					return;
				}
				algorithm = new NSGA(problem, numIter, sigmaShare, epsilon, popSize, crossProb, mutProb, problem.objectivesNumber());
			}
			else{
				algorithm = new PredatorPrey(popSize, problem, numIter, mutProb);
			}
			JOptionPane.showMessageDialog(MMOGui.this, "Zapoinjem raunanje.", "", JOptionPane.INFORMATION_MESSAGE);
			
			frontsFinalResult = algorithm.start();
						
			JOptionPane.showMessageDialog(MMOGui.this, "Algoritam zavrio. Dostupni rezultati.", "", JOptionPane.INFORMATION_MESSAGE);
		}
	};
	
	private Action actionShowResults = new AbstractAction("Rezultati") {
		
		private static final long serialVersionUID = 1L;

		@Override
		public void actionPerformed(ActionEvent arg0) {
			if(frontsFinalResult == null){
				JOptionPane.showMessageDialog(MMOGui.this, "Rezultati nisu dostupni!", "Pogreka", JOptionPane.ERROR_MESSAGE);
			}
			List<DecimalArraySolution> solutions = fromFrontsToList(frontsFinalResult);
			
			MyDataset myDataset = new MyDataset(frontsFinalResult.size(), solutions.size(), solutions);
			Chart.showChart(myDataset);
		}
	};
	
	private List<DecimalArraySolution> fromFrontsToList(List<Front> fronts){
		
		List<DecimalArraySolution> solutions = new ArrayList<>();
		for(Front f : fronts){
			for(DecimalArraySolution s : f.solutions){
				solutions.add(s);
			}
		}
		return solutions;
	}
	
	
	public static void main(String[] args) {
		
		SwingUtilities.invokeLater(new Runnable() {
			
			@Override
			public void run() {
				new MMOGui().setVisible(true);
			}
		});
	}

}
