package hr.fer.zemris.diplomskiseminar.mmo.gui;

import hr.fer.zemris.diplomskiseminar.mmo.datamodel.DecimalArraySolution;

import java.util.List;

import org.jfree.data.DomainInfo;
import org.jfree.data.Range;
import org.jfree.data.RangeInfo;
import org.jfree.data.xy.AbstractXYDataset;
import org.jfree.data.xy.XYDataset;

public class MyDataset extends AbstractXYDataset implements XYDataset, DomainInfo, RangeInfo{

	private static final long serialVersionUID = 1L;
	private int ZNAKICI_COUNT;
	private int ITEM_COUNT;
	private Double[][] xValues;
	private Double[][] yValues;
	private Number domainMax;
	private Number domainMin;
	private Number rangeMax;
	private Number rangeMin;
	private Range domainRange;
	private Range range;
	
	public MyDataset(int znakiciCount, int itemCount, List<DecimalArraySolution> data) {
		xValues = new Double[znakiciCount][itemCount];
		yValues = new Double[znakiciCount][itemCount];
		this.ZNAKICI_COUNT = znakiciCount;
		this.ITEM_COUNT = itemCount;
		fillData(data);
	}
	
	private void fillData(List<DecimalArraySolution> data){
		double minX = Double.MAX_VALUE;
		double maxX = Double.MIN_VALUE;
		double minY = Double.MAX_VALUE;
		double maxY = Double.MIN_VALUE;
		for(int i=0; i<data.size(); i++){
			DecimalArraySolution solution = data.get(i);
			double x = solution.getValue(0);
			double y = solution.getValue(1);
			xValues[solution.rank][i] = x;
			yValues[solution.rank][i] = y;
			if(x<minX) minX = x;
			if(x>maxX) maxX = x;
			if(y<minY) minY = y;
			if(y>maxY) maxY = y;
		}
		domainMin = new Double(minX);
		domainMax = new Double(maxX);
		domainRange = new Range(minX, maxX);
		rangeMin = new Double(minY);
		rangeMax = new Double(maxY);
		range = new Range(minY, maxY);		
	}

	public Number getX(int paramInt1, int paramInt2)
	 {
	   return this.xValues[paramInt1][paramInt2];
	 }

	 public Number getY(int paramInt1, int paramInt2)
	 {
	   return this.yValues[paramInt1][paramInt2];
	 }

	 public int getSeriesCount()
	 {
	   return this.ZNAKICI_COUNT;
	 }

	 @SuppressWarnings("rawtypes")
	public Comparable getSeriesKey(int paramInt)
	 {
	   return "Fronta " + paramInt;
	 }

	 public int getItemCount(int paramInt)
	 {
	   return this.ITEM_COUNT;
	 }

	 public double getDomainLowerBound()
	 {
	   return this.domainMin.doubleValue();
	 }

	 public double getDomainLowerBound(boolean paramBoolean)
	 {
	   return this.domainMin.doubleValue();
	 }

	 public double getDomainUpperBound()
	 {
	   return this.domainMax.doubleValue();
	 }

	 public double getDomainUpperBound(boolean paramBoolean)
	 {
	   return this.domainMax.doubleValue();
	 }

	 public Range getDomainBounds()
	 {
	   return this.domainRange;
	 }

	 public Range getDomainBounds(boolean paramBoolean)
	 {
	   return this.domainRange;
	 }

	 public Range getDomainRange()
	 {
	   return this.domainRange;
	 }

	 public double getRangeLowerBound()
	 {
	   return this.rangeMin.doubleValue();
	 }

	 public double getRangeLowerBound(boolean paramBoolean)
	 {
	   return this.rangeMin.doubleValue();
	 }

	 public double getRangeUpperBound()
	 {
	   return this.rangeMax.doubleValue();
	 }

	 public double getRangeUpperBound(boolean paramBoolean)
	 {
	   return this.rangeMax.doubleValue();
	 }

	 public Range getRangeBounds(boolean paramBoolean)
	 {
	   return this.range;
	 }

	 public Range getValueRange()
	 {
	   return this.range;
	 }

	 public Number getMinimumDomainValue()
	 {
	   return this.domainMin;
	 }

	 public Number getMaximumDomainValue()
	 {
	   return this.domainMax;
	 }

	 public Number getMinimumRangeValue()
	 {
	   return this.domainMin;
	 }

	 public Number getMaximumRangeValue()
	 {
	   return this.domainMax;
	 }

}
